(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[11],{1269:function(t,n,e){var r=e(1306)();t.exports=r},1270:function(t,n,e){var r=e(1309),a=e(117),i=e(260),o=/^[-+]0x[0-9a-f]+$/i,c=/^0b[01]+$/i,s=/^0o[0-7]+$/i,u=parseInt;t.exports=function(t){if("number"==typeof t)return t;if(i(t))return NaN;if(a(t)){var n="function"==typeof t.valueOf?t.valueOf():t;t=a(n)?n+"":n}if("string"!=typeof t)return 0===t?t:+t;t=r(t);var e=c.test(t);return e||s.test(t)?u(t.slice(2),e?2:8):o.test(t)?NaN:+t}},1306:function(t,n,e){var r=e(1307),a=e(505),i=e(1308);t.exports=function(t){return function(n,e,o){return o&&"number"!=typeof o&&a(n,e,o)&&(e=o=void 0),n=i(n),void 0===e?(e=n,n=0):e=i(e),o=void 0===o?n<e?1:-1:i(o),r(n,e,o,t)}}},1307:function(t,n){var e=Math.ceil,r=Math.max;t.exports=function(t,n,a,i){for(var o=-1,c=r(e((n-t)/(a||1)),0),s=Array(c);c--;)s[i?c:++o]=t,t+=a;return s}},1308:function(t,n,e){var r=e(1270),a=1/0;t.exports=function(t){return t?(t=r(t))===a||t===-1/0?17976931348623157e292*(t<0?-1:1):t===t?t:0:0===t?t:0}},1309:function(t,n,e){var r=e(1310),a=/^\s+/;t.exports=function(t){return t?t.slice(0,r(t)+1).replace(a,""):t}},1310:function(t,n){var e=/\s/;t.exports=function(t){for(var n=t.length;n--&&e.test(t.charAt(n)););return n}},1436:function(t,n,e){"use strict";e.d(n,"a",(function(){return f}));var r=e(0),a=e(1269),i=e.n(a),o=e(215),c=e.n(o),s=e(1),u=function(t){var n=t.className,e=t.scriptRunId,r=t.numParticles,a=t.numParticleTypes,o=t.ParticleComponent;return Object(s.jsx)("div",{className:c()(n,"stHidden"),children:i()(r).map((function(t){var n=Math.floor(Math.random()*a);return Object(s.jsx)(o,{particleType:n},e+t)}))})},f=Object(r.memo)(u)},1809:function(t,n,e){"use strict";e.r(n),e.d(n,"default",(function(){return j})),e.d(n,"NUM_FLAKES",(function(){return v}));var r,a=e(0),i=e.p+"static/media/flake-0.beded754.png",o=e.p+"static/media/flake-1.8077dc15.png",c=e.p+"static/media/flake-2.e3f07d06.png",s=e(1436),u=e(156),f=e(10),p=e(52),m=function(t){var n=arguments.length>1&&void 0!==arguments[1]?arguments[1]:0;return Math.random()*(t-n)+n},l=Object(f.a)("img",{target:"e1hbdfkw0"})((function(t){var n=t.theme;return{position:"fixed",top:"".concat(-150,"px"),marginLeft:"".concat(-75,"px"),zIndex:n.zIndices.balloons,left:"".concat(m(90,10),"vw"),animationDelay:"".concat(m(4e3),"ms"),height:"".concat(150,"px"),width:"".concat(150,"px"),animationDuration:"3000ms",animationName:Object(p.c)(r||(r=Object(u.a)(["\n  from {\n    transform:\n      translateY(0)\n      rotateX(","deg)\n      rotateY(","deg)\n      rotateZ(","deg);\n  }\n\n  to {\n    transform:\n      translateY(calc(100vh + ","px))\n      rotateX(0)\n      rotateY(0)\n      rotateZ(0);\n  }\n"])),m(360),m(360),m(360),150),animationTimingFunction:"ease-in",animationDirection:"normal",animationIterationCount:1,opacity:1}}),""),d=e(1),v=100,x=[i,o,c],b=x.length,h=function(t){var n=t.particleType;return Object(d.jsx)(l,{src:x[n]})},g=function(t){var n=t.scriptRunId;return Object(d.jsx)(s.a,{className:"snow",scriptRunId:n,numParticleTypes:b,numParticles:v,ParticleComponent:h})},j=Object(a.memo)(g)}}]);