/*! For license information please see 34.be658f8c.chunk.js.LICENSE.txt */
(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[34],{1822:function(e,t,r){"use strict";r.r(t),r.d(t,"default",(function(){return d}));r(0);var n=r(74),i=r(108),o=r(10),a=r(259),s=r(165),c=Object(o.a)(a.b,{shouldForwardProp:s.a,target:"e17lx80j1"})((function(e){var t=e.theme;return{marginTop:t.spacing.none,marginBottom:t.spacing.none,marginRight:t.spacing.none,marginLeft:t.spacing.none,borderColor:t.colors.fadedText10,borderTopColor:t.colors.primary,flexGrow:0,flexShrink:0}}),""),l=Object(o.a)("div",{target:"e17lx80j0"})((function(e){return{display:"flex",gap:e.theme.spacing.lg,alignItems:"center",width:"100%"}}),""),p=r(1);var d=function(e){var t=e.width,r=e.element,o=Object(n.c)(),a={width:t};return Object(p.jsx)("div",{className:"stSpinner",style:a,children:Object(p.jsxs)(l,{children:[Object(p.jsx)(c,{$size:o.iconSizes.twoXL}),Object(p.jsx)(i.a,{source:r.text,allowHTML:!1})]})})}}}]);