/*! For license information please see 33.aa15e9e2.chunk.js.LICENSE.txt */
(this["webpackJsonpstreamlit-browser"]=this["webpackJsonpstreamlit-browser"]||[]).push([[33],{1829:function(t,r,o){"use strict";o.r(r),o.d(r,"default",(function(){return u}));var e=o(5),a=(o(0),o(74)),c=o(212),n=o(1704),l=o.n(n),i=o(1);var u=Object(c.a)((function(t){var r=t.width,o=t.element,c=t.height,n=function(){return!!c},u=function(t){var l=JSON.parse(t.spec);n()?(l.layout.width=r,l.layout.height=c):o.useContainerWidth&&(l.layout.width=r);var i=Object(a.c)();return l.layout=function(t,r){var o=r.colors,a=r.genericFonts,c={font:{color:o.bodyText,family:a.bodyFont},paper_bgcolor:o.bgColor,plot_bgcolor:o.secondaryBg};return Object(e.a)(Object(e.a)({},t),{},{font:Object(e.a)(Object(e.a)({},c.font),t.font),paper_bgcolor:t.paper_bgcolor||c.paper_bgcolor,plot_bgcolor:t.plot_bgcolor||c.plot_bgcolor})}(l.layout,i),l};switch(o.chart){case"url":return function(t){var o=c||450,e=r;return Object(i.jsx)("iframe",{title:"Plotly",src:t,style:{width:e,height:o}})}(o.url);case"figure":return function(t){var r=JSON.parse(t.config),o=u(t),e=o.data,a=o.layout,c=o.frames;return Object(i.jsx)(l.a,{className:"stPlotlyChart",data:e,layout:a,config:r,frames:c},n()?"fullscreen":"original")}(o.figure);default:throw new Error("Unrecognized PlotlyChart type: ".concat(o.chart))}}))}}]);